@@ -1,10 +1,10 @@
-package URI;  # $Date: 2002/09/03 03:35:23 $
+package URI;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = "1.22";
+$VERSION = "1.51";
 
-use vars qw($ABS_REMOTE_LEADING_DOTS $ABS_ALLOW_RELATIVE_SCHEME);
+use vars qw($ABS_REMOTE_LEADING_DOTS $ABS_ALLOW_RELATIVE_SCHEME $DEFAULT_QUERY_FORM_DELIMITER);
 
 my %implements;  # mapping from scheme to implementor class
 
@@ -22,12 +22,16 @@ use Carp ();
 use URI::Escape ();
 
 use overload ('""'     => sub { ${$_[0]} },
-	      '=='     => sub { overload::StrVal($_[0]) eq
-                                overload::StrVal($_[1])
-                              },
+              '=='     => sub { _obj_eq(@_) },
+              '!='     => sub { !_obj_eq(@_) },
               fallback => 1,
              );
 
+# Check if two objects are the same object
+sub _obj_eq {
+    return overload::StrVal($_[0]) eq overload::StrVal($_[1]);
+}
+
 sub new
 {
     my($class, $uri, $scheme) = @_;
@@ -42,10 +46,12 @@ sub new
     my $impclass;
     if ($uri =~ m/^($scheme_re):/so) {
 	$scheme = $1;
-    } else {
+    }
+    else {
 	if (($impclass = ref($scheme))) {
 	    $scheme = $scheme->scheme;
-	} elsif ($scheme && $scheme =~ m/^($scheme_re)(?::|$)/o) {
+	}
+	elsif ($scheme && $scheme =~ m/^($scheme_re)(?::|$)/o) {
 	    $scheme = $1;
         }
     }
@@ -71,7 +77,8 @@ sub _init
 {
     my $class = shift;
     my($str, $scheme) = @_;
-    $str =~ s/([^$uric\#])/$URI::Escape::escapes{$1}/go;
+    # find all funny characters and encode the bytes.
+    $str = $class->_uric_escape($str);
     $str = "$scheme:$str" unless $str =~ /^$scheme_re:/o ||
                                  $class->_no_scheme_ok;
     my $self = bless \$str, $class;
@@ -79,6 +86,14 @@ sub _init
 }
 
 
+sub _uric_escape
+{
+    my($class, $str) = @_;
+    $str =~ s*([^$uric\#])* URI::Escape::escape_char($1) *ego;
+    return $str;
+}
+
+
 sub implementor
 {
     my($scheme, $impclass) = @_;
@@ -104,7 +119,7 @@ sub implementor
     # preloaded (with 'use') implementation
     $ic = "URI::$scheme";  # default location
 
-    # turn scheme into a valid perl identifier by a simple tranformation...
+    # turn scheme into a valid perl identifier by a simple transformation...
     $ic =~ s/\+/_P/g;
     $ic =~ s/\./_O/g;
     $ic =~ s/\-/_/g;
@@ -159,12 +174,14 @@ sub _scheme
 	my $newself = URI->new("$new:$$self");
 	$$self = $$newself; 
 	bless $self, ref($newself);
-    } else {
+    }
+    else {
 	if ($self->_no_scheme_ok) {
 	    $old = $1 if $$self =~ s/^($scheme_re)://o;
 	    Carp::carp("Oops, opaque part now look like scheme")
 		if $^W && $$self =~ m/^$scheme_re:/o
-	} else {
+	}
+	else {
 	    $old = $1 if $$self =~ m/^($scheme_re):/o;
 	}
     }
@@ -200,7 +217,7 @@ sub opaque
 
     my $new_opaque = shift;
     $new_opaque = "" unless defined $new_opaque;
-    $new_opaque =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+    $new_opaque =~ s/([^$uric])/ URI::Escape::escape_char($1)/ego;
 
     $$self = defined($old_scheme) ? $old_scheme : "";
     $$self .= $new_opaque;
@@ -225,7 +242,7 @@ sub fragment
 
     my $new_frag = shift;
     if (defined $new_frag) {
-	$new_frag =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+	$new_frag =~ s/([^$uric])/ URI::Escape::escape_char($1) /ego;
 	$$self .= "#$new_frag";
     }
     $old;
@@ -239,22 +256,65 @@ sub as_string
 }
 
 
-sub canonical
+sub as_iri
 {
     my $self = shift;
+    my $str = $$self;
+    if ($str =~ /\bxn--/ && $self->can("ihost")) {
+	my $ihost = $self->ihost;
+	if ($ihost) {
+	    my $u = $self->clone;
+	    $u->host("%%host%%");
+	    $str = $u->as_string;
+	    $str =~ s/%%host%%/$ihost/;
+	}
+    }
+    if ($str =~ s/%([89A-F][0-9A-F])/chr(hex($1))/eg) {
+	# All this crap because the more obvious:
+	#
+	#   Encode::decode("UTF-8", $str, sub { sprintf "%%%02X", shift })
+	#
+	# doesn't work.  Apparently passing a sub as CHECK only works
+	# for 'ascii' and similar direct encodings.
 
-    # Make sure scheme is lowercased
+	require Encode;
+	my $enc = Encode::find_encoding("UTF-8");
+	my $u = "";
+	while (length $str) {
+	    $u .= $enc->decode($str, Encode::FB_QUIET());
+	    if (length $str) {
+		# escape next char
+		$u .= URI::Escape::escape_char(substr($str, 0, 1, ""));
+	    }
+	}
+	$str = $u;
+    }
+    return $str;
+}
+
+
+sub canonical
+{
+    # Make sure scheme is lowercased, that we don't escape unreserved chars,
+    # and that we use upcase escape sequences.
+
+    my $self = shift;
     my $scheme = $self->_scheme || "";
     my $uc_scheme = $scheme =~ /[A-Z]/;
-    my $lc_esc    = $$self =~ /%(?:[a-f][a-fA-F0-9]|[A-F0-9][a-f])/;
-    if ($uc_scheme || $lc_esc) {
-	my $other = $self->clone;
-	$other->_scheme(lc $scheme) if $uc_scheme;
-	$$other =~ s/(%(?:[a-f][a-fA-F0-9]|[A-F0-9][a-f]))/uc($1)/ge
-	    if $lc_esc;
-	return $other;
+    my $esc = $$self =~ /%[a-fA-F0-9]{2}/;
+    return $self unless $uc_scheme || $esc;
+
+    my $other = $self->clone;
+    if ($uc_scheme) {
+	$other->_scheme(lc $scheme);
     }
-    $self;
+    if ($esc) {
+	$$other =~ s{%([0-9a-fA-F]{2})}
+	            { my $a = chr(hex($1));
+                      $a =~ /^[$unreserved]\z/o ? $a : "%\U$1"
+                    }ge;
+    }
+    return $other;
 }
 
 # Compare two URIs, subclasses will provide a more correct implementation
@@ -270,6 +330,17 @@ sub eq {
 sub abs { $_[0]; }
 sub rel { $_[0]; }
 
+# help out Storable
+sub STORABLE_freeze {
+       my($self, $cloning) = @_;
+       return $$self;
+}
+
+sub STORABLE_thaw {
+       my($self, $cloning, $str) = @_;
+       $$self = $str;
+}
+
 1;
 
 __END__
@@ -304,19 +375,19 @@ This module implements the C<URI> class.  Objects of this class
 represent "Uniform Resource Identifier references" as specified in RFC
 2396 (and updated by RFC 2732).
 
-A Uniform Resource Identifier is a compact string of characters for
-identifying an abstract or physical resource.  A Uniform Resource
-Identifier can be further classified either a Uniform Resource Locator
+A Uniform Resource Identifier is a compact string of characters that
+identifies an abstract or physical resource.  A Uniform Resource
+Identifier can be further classified as either a Uniform Resource Locator
 (URL) or a Uniform Resource Name (URN).  The distinction between URL
 and URN does not matter to the C<URI> class interface. A
 "URI-reference" is a URI that may have additional information attached
 in the form of a fragment identifier.
 
-An absolute URI reference consists of three parts.  A I<scheme>, a
-I<scheme specific part> and a I<fragment> identifier.  A subset of URI
+An absolute URI reference consists of three parts:  a I<scheme>, a
+I<scheme-specific part> and a I<fragment> identifier.  A subset of URI
 references share a common syntax for hierarchical namespaces.  For
-these the scheme specific part is further broken down into
-I<authority>, I<path> and I<query> components.  These URI can also
+these, the scheme-specific part is further broken down into
+I<authority>, I<path> and I<query> components.  These URIs can also
 take the form of relative URI references, where the scheme (and
 usually also the authority) component is missing, but implied by the
 context of the URI reference.  The three forms of URI reference
@@ -326,7 +397,7 @@ syntax are summarized as follows:
   <scheme>://<authority><path>?<query>#<fragment>
   <path>?<query>#<fragment>
 
-The components that a URI reference can be divided into depend on the
+The components into which a URI reference can be divided depend on the
 I<scheme>.  The C<URI> class provides methods to get and set the
 individual components.  The methods available for a specific
 C<URI> object depend on the scheme.
@@ -337,10 +408,12 @@ The following methods construct new C<URI> objects:
 
 =over 4
 
-=item $uri = URI->new( $str, [$scheme] )
+=item $uri = URI->new( $str )
 
-This class method constructs a new URI object.  The string
-representation of a URI is given as argument together with an optional
+=item $uri = URI->new( $str, $scheme )
+
+Constructs a new URI object.  The string
+representation of a URI is given as argument, together with an optional
 scheme specification.  Common URI wrappers like "" and <>, as well as
 leading and trailing white space, are automatically removed from
 the $str argument before it is processed further.
@@ -349,11 +422,11 @@ The constructor determines the scheme, maps this to an appropriate
 URI subclass, constructs a new object of that class and returns it.
 
 The $scheme argument is only used when $str is a
-relative URI.  It can either be a simple string that
-denotes the scheme, a string containing an absolute URI reference or
+relative URI.  It can be either a simple string that
+denotes the scheme, a string containing an absolute URI reference, or
 an absolute C<URI> object.  If no $scheme is specified for a relative
-URI $str, then $str is simply treated as a generic URI (no scheme
-specific methods available).
+URI $str, then $str is simply treated as a generic URI (no scheme-specific
+methods available).
 
 The set of characters available for building URI references is
 restricted (see L<URI::Escape>).  Characters outside this set are
@@ -361,28 +434,32 @@ automatically escaped by the URI constructor.
 
 =item $uri = URI->new_abs( $str, $base_uri )
 
-This constructs a new absolute URI object.  The $str argument can
-denote a relative or absolute URI.  If relative, then it will be
+Constructs a new absolute URI object.  The $str argument can
+denote a relative or absolute URI.  If relative, then it is
 absolutized using $base_uri as base. The $base_uri must be an absolute
 URI.
 
-=item $uri = URI::file->new( $filename, [$os] )
+=item $uri = URI::file->new( $filename )
 
-This constructs a new I<file> URI from a file name.  See L<URI::file>.
+=item $uri = URI::file->new( $filename, $os )
 
-=item $uri = URI::file->new_abs( $filename, [$os] )
+Constructs a new I<file> URI from a file name.  See L<URI::file>.
 
-This constructs a new absolute I<file> URI from a file name.  See
+=item $uri = URI::file->new_abs( $filename )
+
+=item $uri = URI::file->new_abs( $filename, $os )
+
+Constructs a new absolute I<file> URI from a file name.  See
 L<URI::file>.
 
 =item $uri = URI::file->cwd
 
-This returns the current working directory as a I<file> URI.  See
+Returns the current working directory as a I<file> URI.  See
 L<URI::file>.
 
 =item $uri->clone
 
-This method returns a copy of the $uri.
+Returns a copy of the $uri.
 
 =back
 
@@ -391,16 +468,16 @@ This method returns a copy of the $uri.
 The methods described in this section are available for all C<URI>
 objects.
 
-Methods that give access to components of a URI will always return the
-old value of the component.  The value returned will be C<undef> if the
+Methods that give access to components of a URI always return the
+old value of the component.  The value returned is C<undef> if the
 component was not present.  There is generally a difference between a
 component that is empty (represented as C<"">) and a component that is
 missing (represented as C<undef>).  If an accessor method is given an
-argument it will update the corresponding component in addition to
+argument, it updates the corresponding component in addition to
 returning the old value of the component.  Passing an undefined
-argument will remove the component (if possible).  The description of
-the various accessor methods will tell if the component is passed as
-an escaped or an unescaped string.  Components that can be further
+argument removes the component (if possible).  The description of
+each accessor method indicates whether the component is passed as
+an escaped or an unescaped string.  A component that can be further
 divided into sub-parts are usually passed escaped, as unescaping might
 change its semantics.
 
@@ -408,65 +485,79 @@ The common methods available for all URI are:
 
 =over 4
 
-=item $uri->scheme( [$new_scheme] )
+=item $uri->scheme
 
-This method sets and returns the scheme part of the $uri.  If the $uri is
+=item $uri->scheme( $new_scheme )
+
+Sets and returns the scheme part of the $uri.  If the $uri is
 relative, then $uri->scheme returns C<undef>.  If called with an
-argument, it will update the scheme of $uri, possibly changing the
-class of $uri, and return the old scheme value.  The method croaks
-if the new scheme name is illegal; scheme names must begin with a
+argument, it updates the scheme of $uri, possibly changing the
+class of $uri, and returns the old scheme value.  The method croaks
+if the new scheme name is illegal; a scheme name must begin with a
 letter and must consist of only US-ASCII letters, numbers, and a few
 special marks: ".", "+", "-".  This restriction effectively means
-that scheme have to be passed unescaped.  Passing an undefined
-argument to the scheme method will make the URI relative (if possible).
+that the scheme must be passed unescaped.  Passing an undefined
+argument to the scheme method makes the URI relative (if possible).
 
 Letter case does not matter for scheme names.  The string
 returned by $uri->scheme is always lowercase.  If you want the scheme
 just as it was written in the URI in its original case,
 you can use the $uri->_scheme method instead.
 
-=item $uri->opaque( [$new_opaque] )
+=item $uri->opaque
 
-This method sets and returns the scheme specific part of the $uri 
+=item $uri->opaque( $new_opaque )
+
+Sets and returns the scheme-specific part of the $uri
 (everything between the scheme and the fragment)
 as an escaped string.
 
-=item $uri->path( [$new_path] )
+=item $uri->path
 
-This method sets and returns the same value as $uri->opaque unless the URI
+=item $uri->path( $new_path )
+
+Sets and returns the same value as $uri->opaque unless the URI
 supports the generic syntax for hierarchical namespaces.
 In that case the generic method is overridden to set and return
 the part of the URI between the I<host name> and the I<fragment>.
 
-=item $uri->fragment( [$new_frag] )
+=item $uri->fragment
 
-This method returns the fragment identifier of a URI reference
+=item $uri->fragment( $new_frag )
+
+Returns the fragment identifier of a URI reference
 as an escaped string.
 
 =item $uri->as_string
 
-This method returns a URI object to a plain string.  URI objects are
+Returns a URI object to a plain ASCII string.  URI objects are
 also converted to plain strings automatically by overloading.  This
 means that $uri objects can be used as plain strings in most Perl
 constructs.
 
+=item $uri->as_iri
+
+Returns a Unicode string representing the URI.  Escaped UTF-8 sequences
+representing non-ASCII characters are turned into their corresponding Unicode
+code point.
+
 =item $uri->canonical
 
-This method will return a normalized version of the URI.  The rules
-for normalization are scheme dependent.  It usually involves
-lowercasing of the scheme and the Internet host name components,
+Returns a normalized version of the URI.  The rules
+for normalization are scheme-dependent.  They usually involve
+lowercasing the scheme and Internet host name components,
 removing the explicit port specification if it matches the default port,
 uppercasing all escape sequences, and unescaping octets that can be
 better represented as plain characters.
 
-For efficiency reasons, if the $uri already was in normalized form,
+For efficiency reasons, if the $uri is already in normalized form,
 then a reference to it is returned instead of a copy.
 
 =item $uri->eq( $other_uri )
 
 =item URI::eq( $first_uri, $other_uri )
 
-This method tests whether two URI references are equal.  URI references
+Tests whether two URI references are equal.  URI references
 that normalize to the same string are considered equal.  The method
 can also be used as a plain function which can also test two string
 arguments.
@@ -476,14 +567,14 @@ same object, use the '==' operator.
 
 =item $uri->abs( $base_uri )
 
-This method returns an absolute URI reference.  If $uri already is
+Returns an absolute URI reference.  If $uri is already
 absolute, then a reference to it is simply returned.  If the $uri
 is relative, then a new absolute URI is constructed by combining the
 $uri and the $base_uri, and returned.
 
 =item $uri->rel( $base_uri )
 
-This method returns a relative URI reference if it is possible to
+Returns a relative URI reference if it is possible to
 make one that denotes the same resource relative to $base_uri.
 If not, then $uri is simply returned.
 
@@ -492,114 +583,200 @@ If not, then $uri is simply returned.
 =head1 GENERIC METHODS
 
 The following methods are available to schemes that use the
-common/generic syntax for hierarchical namespaces.  The description of
-schemes below will tell which one these are.  Unknown schemes are
+common/generic syntax for hierarchical namespaces.  The descriptions of
+schemes below indicate which these are.  Unknown schemes are
 assumed to support the generic syntax, and therefore the following
 methods:
 
 =over 4
 
-=item $uri->authority( [$new_authority] )
+=item $uri->authority
 
-This method sets and returns the escaped authority component
+=item $uri->authority( $new_authority )
+
+Sets and returns the escaped authority component
 of the $uri.
 
-=item $uri->path( [$new_path] )
+=item $uri->path
 
-This method sets and returns the escaped path component of
+=item $uri->path( $new_path )
+
+Sets and returns the escaped path component of
 the $uri (the part between the host name and the query or fragment).
-The path will never be undefined, but it can be the empty string.
+The path can never be undefined, but it can be the empty string.
 
-=item $uri->path_query( [$new_path_query] )
+=item $uri->path_query
 
-This method sets and returns the escaped path and query
+=item $uri->path_query( $new_path_query )
+
+Sets and returns the escaped path and query
 components as a single entity.  The path and the query are
 separated by a "?" character, but the query can itself contain "?".
 
-=item $uri->path_segments( [$segment,...] )
+=item $uri->path_segments
 
-This method sets and returns the path.  In scalar context it returns
-the same value as $uri->path.  In list context it will return the
+=item $uri->path_segments( $segment, ... )
+
+Sets and returns the path.  In a scalar context, it returns
+the same value as $uri->path.  In a list context, it returns the
 unescaped path segments that make up the path.  Path segments that
 have parameters are returned as an anonymous array.  The first element
-is the unescaped path segment proper.  Subsequent elements are escaped
+is the unescaped path segment proper;  subsequent elements are escaped
 parameter strings.  Such an anonymous array uses overloading so it can
 be treated as a string too, but this string does not include the
 parameters.
 
-=item $uri->query( [$new_query] )
+Note that absolute paths have the empty string as their first
+I<path_segment>, i.e. the I<path> C</foo/bar> have 3
+I<path_segments>; "", "foo" and "bar".
 
-This method sets and returns the escaped query component of
+=item $uri->query
+
+=item $uri->query( $new_query )
+
+Sets and returns the escaped query component of
 the $uri.
 
-=item $uri->query_form( [$key => $value,...] )
+=item $uri->query_form
 
-This method sets and returns query components that use the
+=item $uri->query_form( $key1 => $val1, $key2 => $val2, ... )
+
+=item $uri->query_form( $key1 => $val1, $key2 => $val2, ..., $delim )
+
+=item $uri->query_form( \@key_value_pairs )
+
+=item $uri->query_form( \@key_value_pairs, $delim )
+
+=item $uri->query_form( \%hash )
+
+=item $uri->query_form( \%hash, $delim )
+
+Sets and returns query components that use the
 I<application/x-www-form-urlencoded> format.  Key/value pairs are
-separated by "&" and the key is separated from the value with a "="
+separated by "&", and the key is separated from the value by a "="
 character.
 
-=item $uri->query_keywords( [$keywords,...] )
+The form can be set either by passing separate key/value pairs, or via
+an array or hash reference.  Passing an empty array or an empty hash
+removes the query component, whereas passing no arguments at all leaves
+the component unchanged.  The order of keys is undefined if a hash
+reference is passed.  The old value is always returned as a list of
+separate key/value pairs.  Assigning this list to a hash is unwise as
+the keys returned might repeat.
 
-This method sets and returns query components that use the
+The values passed when setting the form can be plain strings or
+references to arrays of strings.  Passing an array of values has the
+same effect as passing the key repeatedly with one value at a time.
+All the following statements have the same effect:
+
+    $uri->query_form(foo => 1, foo => 2);
+    $uri->query_form(foo => [1, 2]);
+    $uri->query_form([ foo => 1, foo => 2 ]);
+    $uri->query_form([ foo => [1, 2] ]);
+    $uri->query_form({ foo => [1, 2] });
+
+The $delim parameter can be passed as ";" to force the key/value pairs
+to be delimited by ";" instead of "&" in the query string.  This
+practice is often recommended for URLs embedded in HTML or XML
+documents as this avoids the trouble of escaping the "&" character.
+You might also set the $URI::DEFAULT_QUERY_FORM_DELIMITER variable to
+";" for the same global effect.
+
+The C<URI::QueryParam> module can be loaded to add further methods to
+manipulate the form of a URI.  See L<URI::QueryParam> for details.
+
+=item $uri->query_keywords
+
+=item $uri->query_keywords( $keywords, ... )
+
+=item $uri->query_keywords( \@keywords )
+
+Sets and returns query components that use the
 keywords separated by "+" format.
 
+The keywords can be set either by passing separate keywords directly
+or by passing a reference to an array of keywords.  Passing an empty
+array removes the query component, whereas passing no arguments at
+all leaves the component unchanged.  The old value is always returned
+as a list of separate words.
+
 =back
 
 =head1 SERVER METHODS
 
-Schemes where the I<authority> component denotes a Internet host will
-have the following methods available in addition to the generic
+For schemes where the I<authority> component denotes an Internet host,
+the following methods are available in addition to the generic
 methods.
 
 =over 4
 
-=item $uri->userinfo( [$new_userinfo] )
+=item $uri->userinfo
 
-This method sets and returns the escaped userinfo part of the
+=item $uri->userinfo( $new_userinfo )
+
+Sets and returns the escaped userinfo part of the
 authority component.
 
-For some schemes this will be a user name and a password separated by
+For some schemes this is a user name and a password separated by
 a colon.  This practice is not recommended. Embedding passwords in
 clear text (such as URI) has proven to be a security risk in almost
 every case where it has been used.
 
-=item $uri->host( [$new_host] )
+=item $uri->host
 
-This method sets and returns the unescaped hostname.
+=item $uri->host( $new_host )
+
+Sets and returns the unescaped hostname.
 
 If the $new_host string ends with a colon and a number, then this
-number will also set the port.
+number also sets the port.
 
-=item $uri->port( [ $new_port] )
+For IPv6 addresses the brackets around the raw address is removed in the return
+value from $uri->host.  When setting the host attribute to an IPv6 address you
+can use a raw address or one enclosed in brackets.  The address needs to be
+enclosed in brackets if you want to pass in a new port value as well.
 
-This method sets and returns the port.  The port is simple integer
+=item $uri->ihost
+
+Returns the host in Unicode form.  Any IDNA A-labels are turned into U-labels.
+
+=item $uri->port
+
+=item $uri->port( $new_port )
+
+Sets and returns the port.  The port is a simple integer
 that should be greater than 0.
 
-If no explicit port is specified in the URI, then the default port of
-the URI scheme is returned. If you don't want the default port
+If a port is not specified explicitly in the URI, then the URI scheme's default port
+is returned. If you don't want the default port
 substituted, then you can use the $uri->_port method instead.
 
-=item $uri->host_port( [ $new_host_port ] )
+=item $uri->host_port
 
-This method sets and returns the host and port as a single
-unit.  The returned value will include a port, even if it matches the
-default port.  The host part and the port part is separated with a
-colon; ":".
+=item $uri->host_port( $new_host_port )
+
+Sets and returns the host and port as a single
+unit.  The returned value includes a port, even if it matches the
+default port.  The host part and the port part are separated by a
+colon: ":".
+
+For IPv6 addresses the bracketing is preserved; thus
+URI->new("http://[::1]/")->host_port returns "[::1]:80".  Contrast this with
+$uri->host which will remove the brackets.
 
 =item $uri->default_port
 
-This method returns the default port of the URI scheme that $uri
-belongs to.  For I<http> this will be the number 80, for I<ftp> this
-will be the number 21, etc.  The default port for a scheme can not be
+Returns the default port of the URI scheme to which $uri
+belongs.  For I<http> this is the number 80, for I<ftp> this
+is the number 21, etc.  The default port for a scheme can not be
 changed.
 
 =back
 
-=head1 SCHEME SPECIFIC SUPPORT
+=head1 SCHEME-SPECIFIC SUPPORT
 
-The following URI schemes are specifically supported.  For C<URI>
-objects not belonging to one of these you can only use the common and
+Scheme-specific support is provided for the following URI schemes.  For C<URI>
+objects that do not belong to one of these, you can only use the common and
 generic methods.
 
 =over 4
@@ -611,7 +788,7 @@ of small data items as "immediate" data, as if it had been included
 externally.
 
 C<URI> objects belonging to the data scheme support the common methods
-and two new methods to access their scheme specific components;
+and two new methods to access their scheme-specific components:
 $uri->media_type and $uri->data.  See L<URI::data> for details.
 
 =item B<file>:
@@ -621,7 +798,7 @@ A new RFC 2396 based specification in not available yet, but file URI
 references are in common use.
 
 C<URI> objects belonging to the file scheme support the common and
-generic methods.  In addition they provide two methods to map file URI
+generic methods.  In addition, they provide two methods for mapping file URIs
 back to local file names; $uri->file and $uri->dir.  See L<URI::file>
 for details.
 
@@ -632,8 +809,8 @@ new RFC 2396 based specification in not available yet, but ftp URI
 references are in common use.
 
 C<URI> objects belonging to the ftp scheme support the common,
-generic and server methods.  In addition they provide two methods to
-access the userinfo sub-components: $uri->user and $uri->password.
+generic and server methods.  In addition, they provide two methods for
+accessing the userinfo sub-components: $uri->user and $uri->password.
 
 =item B<gopher>:
 
@@ -642,8 +819,8 @@ The I<gopher> URI scheme is specified in
 as a RFC 2396 based specification.
 
 C<URI> objects belonging to the gopher scheme support the common,
-generic and server methods. In addition they support some methods to
-access gopher specific path components: $uri->gopher_type,
+generic and server methods. In addition, they support some methods for
+accessing gopher-specific path components: $uri->gopher_type,
 $uri->selector, $uri->search, $uri->string.
 
 =item B<http>:
@@ -669,10 +846,24 @@ search operation to perform to retrieve information from an LDAP
 directory.
 
 C<URI> objects belonging to the ldap scheme support the common,
-generic and server methods as well as specific ldap methods; $uri->dn,
+generic and server methods as well as ldap-specific methods: $uri->dn,
 $uri->attributes, $uri->scope, $uri->filter, $uri->extensions.  See
 L<URI::ldap> for details.
 
+=item B<ldapi>:
+
+Like the I<ldap> URI scheme, but uses a UNIX domain socket.  The
+server methods are not supported, and the local socket path is
+available as $uri->un_path.  The I<ldapi> scheme is used by the
+OpenLDAP package.  There is no real specification for it, but it is
+mentioned in various OpenLDAP manual pages.
+
+=item B<ldaps>:
+
+Like the I<ldap> URI scheme, but uses an SSL connection.  This
+scheme is deprecated, as the preferred way is to use the I<start_tls>
+mechanism.
+
 =item B<mailto>:
 
 The I<mailto> URI scheme is specified in RFC 2368.  The scheme was
@@ -681,17 +872,28 @@ individual or service.  It has (in RFC 2368) been extended to allow
 setting of other mail header fields and the message body.
 
 C<URI> objects belonging to the mailto scheme support the common
-methods and the generic query methods.  In addition they support the
-following mailto specific methods: $uri->to, $uri->headers.
+methods and the generic query methods.  In addition, they support the
+following mailto-specific methods: $uri->to, $uri->headers.
+
+Note that the "foo@example.com" part of a mailto is I<not> the
+C<userinfo> and C<host> but instead the C<path>.  This allowed a
+mailto to contain multiple comma-seperated email addresses.
+
+=item B<mms>:
+
+The I<mms> URL specification can be found at L<http://sdp.ppona.com/>
+C<URI> objects belonging to the mms scheme support the common,
+generic, and server methods, with the exception of userinfo and
+query-related sub-components.
 
 =item B<news>:
 
 The I<news>, I<nntp> and I<snews> URI schemes are specified in
-<draft-gilman-news-url-01> and will hopefully be available as a RFC
+<draft-gilman-news-url-01> and will hopefully be available as an RFC
 2396 based specification soon.
 
 C<URI> objects belonging to the news scheme support the common,
-generic and server methods.  In addition they provide some methods to
+generic and server methods.  In addition, they provide some methods to
 access the path: $uri->group and $uri->message.
 
 =item B<nntp>:
@@ -704,7 +906,7 @@ The I<pop> URI scheme is specified in RFC 2384. The scheme is used to
 reference a POP3 mailbox.
 
 C<URI> objects belonging to the pop scheme support the common, generic
-and server methods.  In addition they provide two methods to access the
+and server methods.  In addition, they provide two methods to access the
 userinfo components: $uri->user and $uri->auth
 
 =item B<rlogin>:
@@ -729,7 +931,7 @@ instead of TCP.  The syntax is the same as rtsp.
 
 Information about rsync is available from http://rsync.samba.org.
 C<URI> objects belonging to the rsync scheme support the common,
-generic and server methods.  In addition they provide methods to
+generic and server methods.  In addition, they provide methods to
 access the userinfo sub-components: $uri->user and $uri->password.
 
 =item B<sip>:
@@ -738,7 +940,7 @@ The I<sip> URI specification is described in sections 19.1 and 25
 of RFC 3261.  C<URI> objects belonging to the sip scheme support the
 common, generic, and server methods with the exception of path related
 sub-components.  In addition, they provide two methods to get and set
-I<sip> parameters, $uri->params_form and $uri->params.
+I<sip> parameters: $uri->params_form and $uri->params.
 
 =item B<sips>:
 
@@ -752,23 +954,29 @@ port is different.
 
 =item B<telnet>:
 
-An old speficication of the I<telnet> URI scheme is found in RFC
+An old specification of the I<telnet> URI scheme is found in RFC
 1738. C<URI> objects belonging to the telnet scheme support the
 common, generic and server methods.
 
+=item B<tn3270>:
+
+These URIs are used like I<telnet> URIs but for connections to IBM
+mainframes.  C<URI> objects belonging to the tn3270 scheme support the
+common, generic and server methods.
+
 =item B<ssh>:
 
 Information about ssh is available at http://www.openssh.com/.
 C<URI> objects belonging to the ssh scheme support the common,
-generic and server methods. In addition they provide methods to
+generic and server methods. In addition, they provide methods to
 access the userinfo sub-components: $uri->user and $uri->password.
 
 =item B<urn>:
 
 The syntax of Uniform Resource Names is specified in RFC 2141.  C<URI>
-objects belonging to the urn scheme provide the common methods and the
-methods: $uri->nid and $uri->nss that returns the Namespace Identifier
-and the Namespace Specific String respectively.
+objects belonging to the urn scheme provide the common methods, and also the
+methods $uri->nid and $uri->nss, which return the Namespace Identifier
+and the Namespace-Specific String respectively.
 
 The Namespace Identifier basically works like the Scheme identifier of
 URIs, and further divides the URN namespace.  Namespace Identifier
@@ -782,18 +990,19 @@ method can be used if you want it in its original case.
 =item B<urn>:B<isbn>:
 
 The C<urn:isbn:> namespace contains International Standard Book
-Numbers (ISBNs) and is described in RFC 3187.  C<URI> object belonging
+Numbers (ISBNs) and is described in RFC 3187.  A C<URI> object belonging
 to this namespace has the following extra methods (if the
-Business::ISBN module is available); $uri->isbn,
-$uri->isbn_publisher_code, $uri->isbn_country_code, $uri->isbn_as_ean.
+Business::ISBN module is available): $uri->isbn,
+$uri->isbn_publisher_code, $uri->isbn_group_code (formerly isbn_country_code,
+which is still supported by issues a deprecation warning), $uri->isbn_as_ean.
 
 =item B<urn>:B<oid>:
 
 The C<urn:oid:> namespace contains Object Identifiers (OIDs) and is
-described in RFC 3061.  An object identifier is sequences of digits
-separated by dots.  C<URI> object belonging to this namespace has an
+described in RFC 3061.  An object identifier consists of sequences of digits
+separated by dots.  A C<URI> object belonging to this namespace has an
 additional method called $uri->oid that can be used to get/set the oid
-value.  In list context oid numbers are returned as separate elements.
+value.  In a list context, oid numbers are returned as separate elements.
 
 =back
 
@@ -834,14 +1043,19 @@ examples:
   URI->new("../../../foo")->abs("http://host/a/b")
       ==> "http://host/foo"
 
+=item $URI::DEFAULT_QUERY_FORM_DELIMITER
+
+This value can be set to ";" to have the query form C<key=value> pairs
+delimited by ";" instead of "&" which is the default.
+
 =back
 
 =head1 BUGS
 
-Using regexp variables like $1 directly as argument to the URI methods
-do not work too well with current perl implementations.  I would argue
+Using regexp variables like $1 directly as arguments to the URI methods
+does not work too well with current perl implementations.  I would argue
 that this is actually a bug in perl.  The workaround is to quote
-them. E.g.:
+them. Example:
 
    /(...)/ || die;
    $u->query("$1");
@@ -852,11 +1066,15 @@ As an alternative to this module, the following (official) regular
 expression can be used to decode a URI:
 
   my($scheme, $authority, $path, $query, $fragment) =
-  $uri =~ m|^(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|;
+  $uri =~ m|(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|;
+
+The C<URI::Split> module provides the function uri_split() as a
+readable alternative.
 
 =head1 SEE ALSO
 
-L<URI::file>, L<URI::WithBase>, L<URI::Escape>, L<URI::Heuristic>
+L<URI::file>, L<URI::WithBase>, L<URI::QueryParam>, L<URI::Escape>,
+L<URI::Split>, L<URI::Heuristic>
 
 RFC 2396: "Uniform Resource Identifiers (URI): Generic Syntax",
 Berners-Lee, Fielding, Masinter, August 1998.
@@ -869,14 +1087,14 @@ http://www.w3.org/Addressing/
 
 =head1 COPYRIGHT
 
-Copyright 1995-2002 Gisle Aas.
+Copyright 1995-2009 Gisle Aas.
 
 Copyright 1995 Martijn Koster.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=head1 AUTHORS / ACKNOWLEDGEMENTS
+=head1 AUTHORS / ACKNOWLEDGMENTS
 
 This module is based on the C<URI::URL> module, which in turn was
 (distantly) based on the C<wwwurl.pl> code in the libwww-perl for
